#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
#include <ctype.h>

#include <TransSkel.h>

#define START_X 1
#define START_Y 10
#define XMIN 1
#define YMIN 1
#define XMAX 31
#define YMAX 21

#define SPIDOFF 200
#define WALL 0
#define BLANK 1
#define MAN 2
#define DEADMAN 3
#define DEMON 4
#define DEADDEMON 5
#define XTRAHAPPY 6
#define NUMSPRITES 7
#define SPRITESIZE 20

#define NUMDEMONS 15
#define MAXDEMONS 100

#define MOVEFACTOR 4
#define MAXDIFF 9
#define INSID 128

#define FILEID 129
#define NEWGAME 1
#define ENDGAME 2
#define PREFS 3
#define QUIT 5

#define EDITID 130
#define WINID 128
#define ABOUTID 129

#define PREFSID 128
#define DIFFITEM 6
#define NUMDEMONSITEM 8

#define INPROG 0
#define DEAD 1
#define VIC 2

/* types */

typedef struct {
	int x, y;
	} pos;

typedef struct {
	int x, y;
	char exists;
	} monster;

/* globals */

unsigned char board[ XMAX - XMIN + 2 ][ YMAX - YMIN + 2 ];
CIconHandle sprite[ NUMSPRITES ];
monster demon[ MAXDEMONS ];
pos player;
unsigned moves = 0;
int numdemons = NUMDEMONS;
int newdemons = NUMDEMONS;
int diff = MOVEFACTOR;
Boolean gr = false;
char state;
WindowPtr win;
MenuHandle fileMenu;

/* functions */

void init( void );
void bsetup( void );
void quit( void );
void death( void );
void victory( void );

void putcat( int x, int y, short spriteid );

void moveto( int x, int y );
Boolean inrange( int x, int y );

Boolean movedemons( void );
void demonto( int d, int x, int y );

pascal void dofile( short item );
pascal void dokey( short c, short code, short modifiers );
pascal void redraw( Boolean resized );
pascal void doabout( short item2 );
pascal void adjustmenus( void );
void doprefs( void );

void CStrToStr255( Str255 pstr, char cstr[] );
short Str255ToShort( Str255 str );

/* macros */

#define install_wall(X,Y) board[X][Y]=WALL;
#define install_blank(X,Y) board[X][Y]=BLANK;

/* program */

void main( void )
{
  SkelInit( nil );
  SkelSetMenuHook( adjustmenus );
  SkelSetEventMask( everyEvent - autoKeyMask );
  
  init();
  SkelEventLoop();
  
  quit();
  SkelCleanup();
}

void init( void )
{
	MenuHandle editMenu;
	long feature;

	srand( (unsigned)time( NULL ) );
	
	win = GetNewWindow( WINID, nil, (WindowPtr)-1L );
	
	if( win == nil )
	{
		SysBeep( 10 );
		ExitToShell();
	}
	
	SkelWindow( win, nil, dokey, redraw, nil, nil, nil, nil, false );
	dofile( NEWGAME );
	ShowWindow( win );
	
	SkelApple( "\pAbout Guy Smiley vs. the Monsters", doabout );
	fileMenu = GetMenu( FILEID );
	SkelMenu( fileMenu, dofile, nil, false, false );
	editMenu = GetMenu( EDITID );
	SkelMenu( editMenu, nil, nil, false, false ); 
	DrawMenuBar();
	
	sprite[ WALL ] = GetCIcon( WALL + SPIDOFF );
	sprite[ BLANK ] = GetCIcon( BLANK + SPIDOFF );
	sprite[ MAN ] = GetCIcon( MAN + SPIDOFF );
	sprite[ DEADMAN ] = GetCIcon( DEADMAN + SPIDOFF );
	sprite[ DEMON ] = GetCIcon( DEMON + SPIDOFF );
	sprite[ DEADDEMON ] = GetCIcon( DEADDEMON + SPIDOFF );
	sprite[ XTRAHAPPY ] = GetCIcon( XTRAHAPPY + SPIDOFF );
}

pascal void adjustmenus( void )
{
	if( gr )
		EnableItem( fileMenu, ENDGAME );
	else
		DisableItem( fileMenu, ENDGAME );
}

pascal void dofile( short item )
{
	switch( item )
	{
		case NEWGAME:
			bsetup();
			break;
		case ENDGAME:
			death();
			break;
		case PREFS:
			doprefs();
			break;
		case QUIT:
			SkelStopEventLoop();
			break;
	}
}

void doprefs( void )
{
	DialogPtr dlog;
	GrafPtr savePort;
	ModalFilterUPP filter;
	short item;
	char cstr[ 256 ];
	Str255 str;
	int tempdiff, tempdemons;
	Boolean inDlog;
	
	GetPort (&savePort);
	dlog = GetNewDialog( PREFSID, nil, (WindowPtr)-1L );
	SetPort( dlog );
	
	tempdiff = diff;
	tempdemons = newdemons;

	sprintf( cstr, "%d", tempdiff );
	CStrToStr255( str, cstr );
	SkelSetDlogStr( dlog, DIFFITEM, str );
	sprintf( cstr, "%d", tempdemons );
	CStrToStr255( str, cstr );
	SkelSetDlogStr( dlog, NUMDEMONSITEM, str );

	ShowWindow( dlog );

	inDlog = true;
	while( inDlog )
	{
		//map return/enter
		filter = SkelDlogFilter( nil, true );
		SkelDlogCancelItem( cancel );
		SkelDlogTracksCursor( true );
		ModalDialog( filter, &item );
		SkelRmveDlogFilter();

		if( item == cancel )
			inDlog = false;
		
		if( item == ok )
			if( tempdiff >= 1 && tempdiff <= MAXDIFF && tempdemons >= 1 &&
				tempdemons <= MAXDEMONS )
				inDlog = false;
			else
				SysBeep( 10 );
			 
		SkelGetDlogStr( dlog, DIFFITEM, str );
		tempdiff = Str255ToShort( str );
		SkelGetDlogStr( dlog, NUMDEMONSITEM, str );
		tempdemons = Str255ToShort( str );
	}
	
	DisposeDialog( dlog );
	SetPort( savePort );	

	if( item == ok )
	{
		diff = tempdiff;
		newdemons = tempdemons;
	}
}

pascal void doabout( short item2 )
{
	DialogPtr dlog;
	GrafPtr savePort;
	ModalFilterUPP filter;
	short item;
	Boolean inDlog;
	
	GetPort (&savePort);
	dlog = GetNewDialog( ABOUTID, nil, (WindowPtr)-1L );
	SetPort( dlog );
	ShowWindow( dlog );

	inDlog = true;
	while( inDlog )
	{
		//map return/enter
		filter = SkelDlogFilter( nil, true );
		ModalDialog( filter, &item );
		SkelRmveDlogFilter();

		if( item == ok )
			inDlog = false;
	}
	
	DisposeDialog( dlog );
	SetPort( savePort );	
}

pascal void redraw( Boolean resized )
{
	int i, j;
	
	for( i = 1; i <= XMAX; i++ )
		for( j = 1; j <= YMAX; j++ )
			putcat( i, j, board[ i ][ j ] );
	
	if( state == VIC )
	{
		for( i = 0; i < numdemons; i++ )
			if( demon[ i ].exists )
				putcat( demon[ i ].x, demon[ i ].y, DEADDEMON );
	}
	else
	{
		for( i = 0; i < numdemons; i++ )
			if( demon[ i ].exists )
				putcat( demon[ i ].x, demon[ i ].y, DEMON );	
	}
	
	if( state == INPROG )
		putcat( player.x, player.y, MAN );
	else if( state == DEAD )
		putcat( player.x, player.y, DEADMAN );
	else if( state == VIC )
		putcat( player.x, player.y, XTRAHAPPY );
}

pascal void dokey( short c, short code, short modifiers )
{
	Boolean keyGood;

	if( gr )
	{
		ObscureCursor();
		switch( c )
		{
			case '8':
			case 'e':
			case 'E':
			case 'i':
			case 'I':
				if( inrange( player.x, player.y - 1 ) )
				{
	       			keyGood = true;
					moveto( player.x, player.y - 1 );
	       		}
				break;
			case '4':
			case 's':
			case 'S':
			case 'j':
			case 'J':
				if( inrange( player.x - 1, player.y ) )
				{
					keyGood = true;
					moveto( player.x - 1, player.y );
	        	}
				break;
			case '6':
			case 'f':
			case 'F':
			case 'l':
			case 'L':
				if( inrange( player.x + 1, player.y ) )
				{
	        		keyGood = true;
					moveto( player.x + 1, player.y );
	        	}
				break;
			case '2':
			case 'x':
			case 'X':
			case 'm':
			case 'M':
				if( inrange( player.x, player.y + 1 ) )
				{
	        		keyGood = true;
					moveto( player.x, player.y + 1 );
	        	}
				break;
			case '5':
			case 'd':
			case 'D':
			case 'k':
			case 'K':
				keyGood = true;
	        	break;
		}
	
		if( keyGood )
		{
			moves++;
			if( movedemons() == false )
			{
				putcat( player.x, player.y, DEADMAN );
				death();
			}
		}
		else
			SysBeep( 10 );
	
		if( player.x == XMAX )
			victory();
	}
}

void death( void )
{
	SysBeep( 10 );
	putcat( player.x, player.y, DEADMAN );
	gr = false;
	state = DEAD;
}

void victory( void )
{
	int i;
	
	SysBeep( 10 );
	
	putcat( player.x, player.y, XTRAHAPPY );
	
	for( i = 0; i < numdemons; i++ )
		if( demon[ i ].exists )
			putcat( demon[ i ].x, demon[ i ].y, DEADDEMON );
	
	gr = false;
	state = VIC;
}

Boolean movedemons( void )
{
	int i, j, newx, newy, xx, yy;

	for( i = 0; i < numdemons; i++ )
	{
		if( demon[ i ].exists )
			if( player.x == demon[ i ].x && player.y == demon[ i ].y )
				return( false );

		for( j = 0; j < numdemons; j++ )
			if( demon[ j ].exists && demon[ i ].exists && i != j )
				if( demon[ j ].x == demon[ i ].x && demon[ j ].y == demon[ i ].y )
					demon[ j ].exists = false;

		if( rand() % diff != 0 && demon[ i ].exists )
		{
			xx = abs( player.x - demon[ i ].x );
			yy = abs( player.y - demon[ i ].y );

			newx = demon[ i ].x;
			newy = demon[ i ].y;

			if( yy > xx )
			{
				if( player.y < demon[ i ].y )
					newy = demon[ i ].y - 1;

				else if( player.y > demon[ i ].y )
					newy = demon[ i ].y + 1;
			}
			else if( yy < xx )
			{
				if( player.x < demon[ i ].x )
					newx = demon[ i ].x - 1;

				else if( player.x > demon[ i ].x )
					newx = demon[ i ].x + 1;
			}

			if( inrange( newx, newy ) )
				demonto( i, newx, newy );

			if( demon[ i ].exists )
				if( player.x == demon[ i ].x && player.y == demon[ i ].y )
					return( false );

			for( j = 0; j < numdemons; j++ )
				if( demon[ j ].exists && i != j )
					if( demon[ j ].x == demon[ i ].x && demon[ j ].y == demon[ i ].y )
						demon[ j ].exists = false;
		}
	}

  return( true );
}

void moveto( int x, int y )
{
	putcat( player.x, player.y, BLANK );
	player.x = x;
	player.y = y;
	putcat( player.x, player.y, MAN );
}

void demonto( int d, int x, int y )
{
	putcat( demon[ d ].x, demon[ d ].y, BLANK );
	demon[ d ].x = x;
	demon[ d ].y = y;
	putcat( demon[ d ].x, demon[ d ].y, DEMON );
}

Boolean inrange( int x, int y )
{
	if( ( x < XMIN ) || ( x > XMAX ) || ( y < YMIN ) || ( y > YMAX ) )
		/* out of range */
	{
		return( false );
	}
	else if( board[ x ][ y ] == WALL )
		/* in wall */
	{
		return( false );
	}
	else
		/* all's well! */
		return( true );
}

void bsetup( void )
{
	int i, x, y;

	gr = true;
	state = INPROG;
	numdemons = newdemons;

	for( x = XMIN; x <= XMAX; x++ )
		for( y = YMIN; y <= YMAX; y++ )
			install_blank( x, y );

	for( i = 0; i <= 50; i++ )
	{
		x = rand() % ( XMAX - XMIN + 1 ) + XMIN;
		y = rand() % ( YMAX - YMIN + 1 ) + YMIN;

		install_wall( x, y );
	}

	for( i = 0; i < numdemons; i++ )
	{
		do {
			x = rand() % ( XMAX - XMIN + 1 ) + XMIN;
			y = rand() % ( YMAX - YMIN + 1 ) + YMIN;
		} while( !inrange( x, y ) );

		demon[ i ].exists = true;
		demon[ i ].x = x;
		demon[ i ].y = y;
	}

	install_blank( START_X, START_Y );
	player.x = START_X;
	player.y = START_Y;
	InvalRect( &(win->portRect) );
}

void putcat( int x, int y, short spriteid )
{
	Rect iconrect;
		
	x--;
	x *= SPRITESIZE;
	y--;
	y *= SPRITESIZE;
	
	iconrect.top = y;
	iconrect.bottom = y + SPRITESIZE;
	iconrect.left = x;
	iconrect.right = x + SPRITESIZE;
	
	PlotCIcon( &iconrect, sprite[ spriteid ] );
}

void quit( void )
{
	DisposeCIcon( sprite[ WALL ] );	
	DisposeCIcon( sprite[ BLANK ] );	
	DisposeCIcon( sprite[ MAN ] );	
	DisposeCIcon( sprite[ DEADMAN ] );	
	DisposeCIcon( sprite[ DEMON ] );
	DisposeCIcon( sprite[ DEADDEMON ] );
	DisposeCIcon( sprite[ XTRAHAPPY ] );
}

void CStrToStr255( Str255 pstr, char cstr[] )
{
	short i;
	
	pstr[ 0 ] = strlen( cstr );
	
	for( i = 1; i <= pstr[ 0 ]; i++ )
		pstr[ i ] = cstr[ i - 1 ];
}

short Str255ToShort( Str255 str )
{
	short i = 0, numbegin, strloc = 1, place = 1;
	Boolean negative = false;
	
	while( isspace( str[ strloc ] ) )
		strloc++;
	
	if( str[ strloc ] == '-' )
	{
		strloc++;
		negative = true;
	}
	else if( str[ strloc ] == '+' )
	{
		strloc++;
		negative = false;
	}
	
	numbegin = strloc;
	strloc = str[ 0 ];
	
	while( strloc >= numbegin )
	{
		i += ( str[ strloc ] - '0' ) * place;
		place *= 10;
		strloc--;
	}
	
	if( negative )
		i *= -1;
	
	return( i );
}
